/*----------------------------------------------------------------------------------------------------------------------*/
/* Powder Structure Solution Program header file */
/*----------------------------------------------------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------------------------------------- ----------*/
/* CONSTANTS THAT SPECIFY MAXIMUM SIZE OF THE PROBLEM AND OTHER CONSTANTS  */
/*----------------------------------------------------------------------------------------------------------------------*/


#define NINTENS 300      /* maximum number of reflections read in */
#define NMOL 5           /* maximum number of independent molecules */
#define NATINMOL 103     /* maximum number of atoms in each molecule */
#define NATS 200         /* maximum total number of atoms in irreducible cell of structure */
#define MAXATNUM 76      /* largest atomic number handled in program + 1 */
#define NATOMTYPE 10     /* maximum number of atom types */
#define NPAR 100         /* maximum number of parameters adjusted in simulated annealing */
#define NSYM 16          /* maximum number of symmetry elements */
#define NLONG 3200       // NATS * NSYM
#define MAXBUF 200       // length of input buffer (also used in pluck_buf)
#define MAXPUTCMD 100    // number of command lines to describe structure
#define NOVLP 5          // maximum number of overlapping reflections  to consider.

#define RADIAN 57.29577951
#define TWOPI 6.283185307
#define BIG 2147483648.  /* 2^31 - largest output from rand() */
#define ALMOSTBIG 1073741824. /* 2^30 */


/*---------------------------------------------------------------------------------------------------------------------*/
/* STRUCTURES */
/*---------------------------------------------------------------------------------------------------------------------*/


struct intens {
  int i_hkl[3];          /* hkl indices */
  int i_m;               /* multiplicity */
  float i_iobs;          /* observed intensity */
  float i_tth;           /* 2theta */
  float i_wid;            /* LeBail (FULLPROF) hwhm */
  float i_fflp[NATOMTYPE];    /* form factor/sqrt(Lorentz-Polarization factor)  */
  float i_cij[NOVLP];     /* Cij - overlap coefficient */
  float i_icalc;         /* current calculated intensity */
  float i_dif;         /* Icalculated - Iobserved  */
} intens[NINTENS];


struct atom {           /* refers to atoms in the irreducible cell */
  int a_atnum;           /* atomic number */
  int a_atype;           /* atom type to calculate the scattering factors (not atomic number) */
  float a_frac;          /* fractional occupancy */
  char a_symbol[10];     /* atomic symbol */
  float a_xyz[3];        /* fractional coordinates for each atom in entire cell */
} atoms[NATS];

struct molecule {
  int m_natoms;	// number of atoms in this molecule
  struct atom m_atom[NATINMOL]; // each atom, actually has cartesian coordinates
} molecule[NMOL];

struct descr {
  float d_f[NPAR];       /* parameter values */
  float d_r;             /* calculated r factor */
};

struct desc_range { // range of each descriptive parameter
  float r_fmin[NPAR];
  float r_fmax[NPAR];
} d_range;

struct symelement {
  float s_m[3][3];
  float s_t[3];
} symelement[NSYM];

struct pt_rec
{
  int x;         /*parsing sym element x y and z components*/
  int y;
  int z;
};

struct symxyz
{
  char X[20];
  char Y[20];
  char Z[20];
};

/*----------------------------------------------------------------------------------------------------------------------*/
/* EXTERNAL VARIABLES */
/*----------------------------------------------------------------------------------------------------------------------*/


extern FILE *outf;

extern float lambda;                          /* x-ray wavelength */ 
extern float cellconv[3][3];   /* contains the matrix to transform cartesian coords to fractional lattice */
extern int nsym;                              /* actual number of symmetry operations, >= 1 */
extern int centrosym;                         /* 1 if centrosymmetric, 0 otherwise */
extern float cella, cellb, cellc, alpha, beta, gama ;     /* cell parameters  */
extern char space_group_symbol[10];               /* Space group symbol */
extern int nintens;                           /* number of integrated intensities read */
extern float sumiobs;                         /* sum of the integrated intensities read */
extern int nats;                              /* total number of atoms */
extern char title[100];                       /* title */
extern char putcmd[MAXPUTCMD][MAXBUF];
extern int npar;/* number of adjusted parameters*/
extern struct molecule molecule[NMOL];
extern struct atom atoms[NATS];
extern struct desc_range d_range;
extern struct symelement symelement[NSYM];
extern struct intens intens[NINTENS];

/*--------------------------------------------------------------------------------------------------------------------*/
/* PROTOTYPES */
/*----------------------------------------------------------------------------------------------------------------------*/

void lattconv(void);
void symsetup(FILE *);
void moldraw_file(char *);
void schakal_file(char *);
void cif_file(char *);
void fullprof_file(char *);
void ortep3_file(char *);
void gsas_file(char *);
void putatoms(struct descr *);
float fflp(int, float);
void dcopy(struct descr *, struct descr *);
void printpars(struct descr *);

void atcopy(struct atom *, struct atom *);
void init_pluck(char *);
char *pluck(void);
void setup_rot(float[3][3]);
void rot_axis(char *, struct descr *);
void rot_body_fix(char *, int);
void rot_body_var(char *, int, struct descr *);
void putmol(char *, int, struct descr *);
int norm_vec(float[3]);
void errout(char *);
float rfactor(void);
void costabstuff(void);
float trans_parse (char[]);
void rot_parse (char[], struct pt_rec *);
void invert_string(char[], char[]);













